@@ -26,7 +26,6 @@ program wt_diff2
  !---- ѿ ----
   real(8), dimension(im,jm,0:km)     :: xyz_Torvel   ! ȥ®
   real(8), dimension((nm+1)**2,0:lm) :: wt_Torvel    ! ȥ®
-  real(8), dimension((nm+1)**2,0:lm) :: wt_Torvelr   ! ȥ®/r
 
  !---- ʬѥ᥿ ----
   real(8), parameter :: dt=1e-5                 ! ֥ƥå״ֳ
@@ -60,7 +59,7 @@ program wt_diff2
   do it=1,nt                                         ! Euler ˡˤʬ
      wt_Torvel = wt_Torvel + dt *( nu * wt_Lapla_wt(wt_Torvel))
 
-     call wt_TorBoundaries(wt_Torvel,TvBC)
+     call wt_TorBoundaries(wt_Torvel,cond=TvBC)
 
      if(mod(it,ndisp) .eq. 0)then                    ! 
         xyz_Torvel = xyz_wt(wt_Torvel)
