@@ -6,12 +6,13 @@
 !      ݥݥƥ󥷥Ȼ
 !
 !  2002/06/10  ݹ
+!      2004/02/15   ʿ gt4f90io λѤȼ饤֥̾ѹ
 !
 program wt_diff5
 
   use lumatrix
   use wt_module
-  use gtool_history
+  use gt4_history
   implicit none
 
  !---- ֲ ----
