@@ -45,14 +45,14 @@ module w_module
   public y_AvrLon_xy, AvrLon_x                ! ʿ    
   public x_AvrLat_xy, AvrLat_y                ! ʿ    
 
-  public nm_EnergyFromStreamfunction_w      ! ͥ륮ڥȥ           
-                                            ! (ʿȿ n, Ӿȿ m )
-  public n_EnergyFromStreamfunction_w       ! ͥ륮ڥȥ
-                                            ! (ʿȿ n ) 
-  public nm_EnstrophyFromStreamfunction_w   ! 󥹥ȥեڥȥ     
-                                            ! (ʿȿ n, Ӿȿ m )
-  public n_EnstrophyFromStreamfunction_w    ! 󥹥ȥեڥȥ  
-                                            !  (ʿȿ n )
-  public w_spectrum_VMiss                   ! »
+  public nm_EnergyFromStreamfunc_w            ! ͥ륮ڥȥ           
+                                              ! (ʿȿ n, Ӿȿ m )
+  public n_EnergyFromStreamfunc_w             ! ͥ륮ڥȥ
+                                              ! (ʿȿ n ) 
+  public nm_EnstrophyFromStreamfunc_w         ! 󥹥ȥեڥȥ     
+                                              ! (ʿȿ n, Ӿȿ m )
+  public n_EnstrophyFromStreamfunc_w          ! 󥹥ȥեڥȥ  
+                                              !  (ʿȿ n )
+  public w_spectrum_VMiss                     ! »
 
 end module w_module
