@@ -1,16 +1,18 @@
 !----------------------------------------------------------------------
-!     Copyright (c) 2002 Shin-ichi Takehiro. All rights reserved.
+!     Copyright (c) 2002-2005 Shin-ichi Takehiro. All rights reserved.
 !----------------------------------------------------------------------
 !ɽ  wa_base_module
 !
 !  2002/02/02  ݹ  ¿Ѥ˲¤
 !      2002/03/30  ݹ  ⥸塼̾ѹ
 !      2002/05/25  ݹ  ʻɸ̤٤̿̾ˡѹ
+!      2005/01/09  ݹ  msgdmp -> MessageNotify ѹ
 !
 !         Ѵʻǡ, ڥȥǡ礭Ϸᤦ
 !
 module wa_base_module
 
+  use dc_message
   use w_base_module, only : im, jm, nm, it, t, y, ip, p, r, ia, a
   implicit none
 
@@ -83,7 +85,7 @@ module wa_base_module
 
       k= size(wa_data,2)
       if  ( k > km ) then
-         call msgdmp('E','xya_wa','Size of 3rd dimension invalid.')
+         call MessageNotify('E','xya_wa','Size of 3rd dimension invalid.')
       else
          call snts2g(nm,im,im,jm,jm,size(wa_data,2),wa_data,xya_wa,&
               it,t,y,ipk(1:k,:),pk(1:k,:),rk(1:k,:),ia,a,q,ws,ww,ipval,ifval)
@@ -117,7 +119,7 @@ module wa_base_module
 
       k = size(xya_data,3)
       if ( k > km ) then
-         call msgdmp('E','wa_xya','Size of 3rd dimension invalid.')
+         call MessageNotify('E','wa_xya','Size of 3rd dimension invalid.')
       else
          call sntg2s(nm,im,im,jm,jm,k,xya_data,wa_xya,&
               it,t,y,ipk(1:k,:),pk(1:k,:),rk(1:k,:),ia,a,q,ws,ww,ipval,ifval)
