@@ -65,23 +65,23 @@ module wa_module
   public ya_AvrLon_xya, a_AvrLon_xa           ! ʿ    
   public xa_AvrLat_xya, a_AvrLat_ya           ! ʿ    
 
-  public nma_EnergyFromStreamfunction_wa      ! ͥ륮ڥȥ
+  public nma_EnergyFromStreamfunc_wa          ! ͥ륮ڥȥ
                                               ! (ʿȿ n, Ӿȿ m )
-  public na_EnergyFromStreamfunction_wa       ! ͥ륮ڥȥ
+  public na_EnergyFromStreamfunc_wa           ! ͥ륮ڥȥ
                                               ! (ʿȿ n )
-  public nma_EnstrophyFromStreamfunction_wa   ! 󥹥ȥեڥȥ
+  public nma_EnstrophyFromStreamfunc_wa       ! 󥹥ȥեڥȥ
                                               ! (ʿȿ n, Ӿȿ m )
-  public na_EnstrophyFromStreamfunction_wa    ! 󥹥ȥեڥȥ
+  public na_EnstrophyFromStreamfunc_wa        ! 󥹥ȥեڥȥ
                                               !  (ʿȿ n )
   public wa_spectrum_VMiss                    ! »
 
-  public nm_EnergyFromStreamfunction_w      ! ͥ륮ڥȥ           
+  public nm_EnergyFromStreamfunc_w          ! ͥ륮ڥȥ           
                                             ! (ʿȿ n, Ӿȿ m )
-  public n_EnergyFromStreamfunction_w       ! ͥ륮ڥȥ
+  public n_EnergyFromStreamfunc_w           ! ͥ륮ڥȥ
                                             ! (ʿȿ n ) 
-  public nm_EnstrophyFromStreamfunction_w   ! 󥹥ȥեڥȥ     
+  public nm_EnstrophyFromStreamfunc_w       ! 󥹥ȥեڥȥ     
                                             ! (ʿȿ n, Ӿȿ m )
-  public n_EnstrophyFromStreamfunction_w    ! 󥹥ȥեڥȥ  
+  public n_EnstrophyFromStreamfunc_w        ! 󥹥ȥեڥȥ  
                                             !  (ʿȿ n )
   public w_spectrum_VMiss                   ! »
 
