@@ -1,5 +1,5 @@
 !----------------------------------------------------------------------
-!     Copyright (c) 2002 Shin-ichi Takehiro. All rights reserved.
+!     Copyright (c) 2002-2005 Shin-ichi Takehiro. All rights reserved.
 !----------------------------------------------------------------------
 !ɽ  wt_module
 !
@@ -9,11 +9,33 @@
 !      2002/11/10  ݹ  ¶֤Ǥζ롼ɲ
 !      2002/11/24  ݹ  ӥշϤ
 !                            ץ饷ղ򤭥롼ɲ
+!      2002/11/28  ݹ  VGradV η׻ѹ.
+!      2005/01/09  ݹ  msgdmp -> MessageNotify ѹ
+!      2005/01/09  ݹ  ٥ȥβžγʬ׻ؿɲ.
+!      2005/02/19  ݹ  ȥǴ嶭ˤ®پ
+!                             Ϳ褦˥ץɲ
+!      2005/02/19  ݹ  ѿ xy_Lon, xy_Lat ɲ
+!      2005/03/18  ݹ  ʬʿѴؿɲ
+!                            z_IntLon_xz, x_IntRad_xz, IntLonRad_xz
+!                            z_InRad_xz, y_IntRad_yz, IntLatRad_yz
+!                            IntRad_z 
+!                            z_IntLon_xz, x_IntRad_xz, IntLonRad_xz
+!                            z_InRad_xz, y_IntRad_yz, IntLatRad_yz
+!                            IntRad_z 
+!      2005/04/24  ݹ  ڥȥϷ׻롼ɲ
+!                            nmz_ToroidalEnergySpectrum_wt
+!                            nz_ToroidalEnergySpectrum_wt
+!                            nmz_PoloidalEnergySpectrum_wt
+!                            nz_PoloidalEnergySpectrum_wt
+!                            ѿ» wt_VMiss ɲ
 !
 !
 !      ǡ index
 !        x :          y :         z : ư
-!        w : Ĵ´ؿڥȥ       t : ӥմؿڥȥ
+!        w : Ĵ´ؿڥȥ
+!        n : Ĵ´ؿڥȥ(ʿȿ)
+!        m : Ĵ´ؿڥȥ(Ӿȿ)
+!        t : ӥմؿڥȥ
 !        a : Ǥդμ
 !
 !        xyz : 3 ʻǡ
@@ -26,6 +48,7 @@
 !
 module wt_module
 
+  use dc_message
   use lumatrix
   use wa_module
   use at_module, z_RAD => g_X, z_RAD_WEIGHT => g_X_WEIGHT, &
@@ -40,29 +63,46 @@ module wt_module
   public y_Lat, y_Lat_Weight
   public z_Rad, z_Rad_Weight
   public l_nm, nm_l
+  public xy_Lon, xy_Lat
   public xyz_Lon, xyz_Lat, xyz_Rad
   public wz_Rad
+  public wt_VMiss
 
   public w_xy, xy_w
   public at_Dr_at, t_Dr_t, az_at, at_az
   public xyz_wt, wt_xyz, xyz_wz, wz_xyz, wz_wt, wt_wz
   public wt_DRad_wt, wt_DivRad_wt, wt_RotRad_wt, wt_Lapla_wt
   public xyz_GradLon_wt, xyz_gradlat_wt
-  public wt_DivLon_xyz, wt_DivLat_xyz, wt_Div_xyz_xyz_xyz
+  public wt_DivLon_xyz, wt_DivLat_xyz
+  public wt_Div_xyz_xyz_xyz, xyz_Div_xyz_xyz_xyz
+  public xyz_RotLon_wt_wt, xyz_RotLat_wt_wt, wt_RotRad_xyz_xyz
 
   public yz_IntLon_xyz, xz_IntLat_xyz, xy_IntRad_xyz
   public x_IntLatRad_xyz, y_IntLonRad_xyz, z_IntLonLat_xyz
   public IntLonLatRad_xyz
 
+  public x_IntLat_xy, y_IntLon_xy, IntLonLat_xy
+  public z_IntLat_yz, y_IntRad_yz, IntLatRad_yz
+  public z_IntLon_xz, x_IntRad_xz, IntLonRad_xz
+  public IntLon_x, IntLat_y, IntRad_z
+
   public yz_AvrLon_xyz, xz_AvrLat_xyz, xy_AvrRad_xyz
   public x_AvrLatRad_xyz, y_AvrLonRad_xyz, z_AvrLonLat_xyz
   public AvrLonLatRad_xyz
 
+  public x_AvrLat_xy, y_AvrLon_xy, AvrLonLat_xy
+  public z_AvrLat_yz, y_AvrRad_yz, AvrLatRad_yz
+  public z_AvrLon_xz, x_AvrRad_xz, AvrLonRad_xz
+  public AvrLon_x, AvrLat_y, AvrRad_z
+
   public wt_KxRGrad_wt, xyz_KGrad_wt, wt_L2_wt, wt_L2Inv_wt, wt_QOperator_wt
   public wt_RadRot_xyz_xyz, wt_RadRotRot_xyz_xyz_xyz
-  public wt_Potential2vector
+  public wt_Potential2vector, wt_Potential2Rotation
   public wt_VGradV
 
+  public nmz_ToroidalEnergySpectrum_wt, nz_ToroidalEnergySpectrum_wt
+  public nmz_PoloidalEnergySpectrum_wt, nz_PoloidalEnergySpectrum_wt
+
   public wt_Boundaries, wt_TorBoundaries, wz_LaplaPol2Pol_wz
   public wt_TormagBoundaries, wt_PolmagBoundaries
 
@@ -96,6 +136,8 @@ module wt_module
   real(8), dimension(:,:,:), allocatable :: xyz_LON, xyz_LAT, xyz_RAD ! ɸ
   real(8), dimension(:,:), allocatable   :: wz_RAD                    ! ɸ
 
+  real(8) :: wt_VMiss = -999.0        ! »
+
   save im, jm, km, nm, lm, ri, ro
 
   contains
@@ -198,7 +240,7 @@ module wt_module
     end function wt_DivRad_wt
 
     function wt_RotRad_wt(wt)    ! žưʬ 
-                                 ! 1/r /r r = /r + 1/r 
+                                 ! 1/r (r.)/r = ./r + 1x./r 
       real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) :: wt
       real(8), dimension((nm+1)*(nm+1),0:lm)             :: wt_RotRad_wt
 
@@ -263,8 +305,70 @@ module wt_module
 
     end function wt_Div_xyz_xyz_xyz
 
-  !--------------- ʬ׻ -----------------
+    function xyz_Div_xyz_xyz_xyz(xyz_Vlon,xyz_Vlat,xyz_Vrad) ! ȯ
+      !
+      ! ˤð򤹤뤿˥٥ȥ cos/r νŤߤ򤫤
+      ! ׻Ƥ. 
+      !      div V = (r/cos)div (Vcos/r) + V_tan/r + V_r/r
+      ! 
+      real(8), dimension(im,jm,0:km), intent(in) :: xyz_Vlon   ! ʬ
+      real(8), dimension(im,jm,0:km), intent(in) :: xyz_Vlat   ! ʬ
+      real(8), dimension(im,jm,0:km), intent(in) :: xyz_Vrad   ! ưʬ
+      real(8), dimension(im,jm,0:km)             :: xyz_Div_xyz_xyz_xyz
 
+      xyz_Div_xyz_xyz_xyz &
+           = xyz_Rad/cos(xyz_Lat) &
+                * xyz_wt(wt_Div_xyz_xyz_xyz(xyz_VLon*cos(xyz_Lat)/xyz_Rad,  &
+                                            xyz_VLat*cos(xyz_Lat)/xyz_Rad,  &
+                                            xyz_VRad*cos(xyz_Lat)/xyz_Rad ))&
+             + xyz_VLat*tan(xyz_Lat)/xyz_Rad &
+             + xyz_VRad/xyz_Rad
+
+    end function xyz_Div_xyz_xyz_xyz
+
+    function xyz_RotLon_wt_wt(wt_Vrad,wt_Vlat) 
+      ! ٥ȥβžηʬ
+      ! 1/rcosաVrad/ߦ-1/r (r Vlat)/r
+
+        real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) :: wt_Vrad ! ưʬ
+        real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) :: wt_Vlat ! ʬ
+
+        real(8), dimension(im,jm,0:km)                     :: xyz_RotLon_wt_wt
+
+        xyz_RotLon_wt_wt =   xyz_GradLon_wt(wt_Vrad) &
+                           - xyz_wt(wt_RotRad_wt(wt_Vlat))
+
+    end function xyz_RotLon_wt_wt
+
+    function xyz_RotLat_wt_wt(wt_Vlon,wt_Vrad) 
+       ! ٥ȥβžΰʬ
+       ! 1/r (r Vlon)/r -  1/r Vrad/ߦ
+
+        real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) :: wt_Vlon ! ʬ
+        real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) :: wt_Vrad ! ưʬ
+
+        real(8), dimension(im,jm,0:km)                     :: xyz_RotLat_wt_wt
+
+        xyz_RotLat_wt_wt =   xyz_wt(wt_RotRad_wt(wt_Vlon)) &
+                           - xyz_GradLat_wt(wt_Vrad) 
+
+    end function xyz_RotLat_wt_wt
+
+    function wt_RotRad_xyz_xyz(xyz_Vlat,xyz_Vlon) 
+      ! ٥ȥβžưʬ
+      ! 1/rcosաVlat/ߦ - 1/rcosա(Vlon cos)/ߦ
+
+        real(8), dimension(im,jm,0:km), intent(in) :: xyz_Vlat
+        real(8), dimension(im,jm,0:km), intent(in) :: xyz_Vlon
+        real(8), dimension((nm+1)*(nm+1),0:lm)     :: wt_RotRad_xyz_xyz
+
+        wt_RotRad_xyz_xyz =   wt_DivLon_xyz(xyz_Vlat) &
+                            - wt_DivLat_xyz(xyz_Vlon)
+
+    end function wt_RotRad_xyz_xyz
+
+  !--------------- ʬ׻ -----------------
+    !----(ϥǡ xyz)---
     function yz_IntLon_xyz(xyz)  ! (Ӿ)ʬ
       real(8), dimension(im,jm,0:km), intent(in) :: xyz    ! 3 ʻǡ
       real(8), dimension(jm,0:km)  :: yz_IntLon_xyz        ! Ҹ̳ʻǡ
@@ -360,8 +464,96 @@ module wt_module
       enddo
     end function IntLonLatRad_xyz
 
-  !--------------- ʿѷ׻ -----------------
+    !----(ϥǡ yz)---
+    function z_IntLat_yz(yz)  ! ʬ
+      real(8), dimension(jm,0:km), intent(in) :: yz   ! 2 ʻǡ
+      real(8), dimension(0:km)  :: z_IntLat_yz        ! ư³ʻǡ
+      integer :: j
 
+      z_IntLat_yz = 0.0d0
+      do j=1,jm
+         z_IntLat_yz(:) = z_IntLat_yz(:) + yz(j,:) * y_Lat_Weight(j)
+      enddo
+    end function z_IntLat_yz
+
+    function y_IntRad_yz(yz)  ! ưʬ
+      real(8), dimension(jm,0:km), intent(in) :: yz    ! 2 ʻǡ
+      real(8), dimension(jm)  :: y_IntRad_yz           ! ٱ߳ʻǡ
+      integer :: k
+
+      y_IntRad_yz = 0.0d0
+      do k=0,km
+         y_IntRad_yz(:) = y_IntRad_yz(:) &
+                       + yz(:,k) * z_Rad_Weight(k) 
+      enddo
+    end function y_IntRad_yz
+
+    function IntLatRad_yz(yz)  ! ư(Ҹ)ʬ
+      real(8), dimension(jm,0:km), intent(in) :: yz  ! Ҹ̳ʻǡ
+      real(8)                   :: IntLatRad_yz      ! ʬ
+      integer :: j, k
+
+      IntLatRad_yz = 0
+      do k=0,km
+         do j=1,jm
+            IntLatRad_yz = IntLatRad_yz &
+                 + yz(j,k) * y_Lat_Weight(j) * z_Rad_Weight(k)
+         enddo
+      enddo
+    end function IntLatRad_yz
+
+    !----(ϥǡ xz)---
+    function z_IntLon_xz(xz)  ! (Ӿ)ʬ
+      real(8), dimension(im,0:km), intent(in) :: xz   ! 2 ʻǡ
+      real(8), dimension(0:km)  :: z_IntLon_xz        ! ľʻǡ
+      integer :: i
+
+      z_IntLon_xz = 0.0d0
+      do i=1,im
+         z_IntLon_xz(:) = z_IntLon_xz(:) + xz(i,:) * x_Lon_Weight(i)
+      enddo
+    end function z_IntLon_xz
+
+    function x_IntRad_xz(xz)  ! ưʬ
+      real(8), dimension(im,0:km), intent(in) :: xz    ! 2 ʻǡ
+      real(8), dimension(im)  :: x_IntRad_xz           ! ٳʻǡ
+      integer :: k
+
+      x_IntRad_xz = 0.0d0
+      do k=0,km
+         x_IntRad_xz(:) = x_IntRad_xz(:) &
+                       + xz(:,k) * z_Rad_Weight(k) 
+      enddo
+    end function x_IntRad_xz
+
+    function IntLonRad_xz(xz)  ! ư(ٱ)ʬ
+      real(8), dimension(im,0:km), intent(in) :: xz    ! 2 ʻǡ
+      real(8)                                 :: IntLonRad_xz      ! ʬ
+      integer :: i, k
+
+      IntLonRad_xz = 0
+      do k=0,km
+         do i=1,im
+            IntLonRad_xz = IntLonRad_xz &
+                 + xz(i,k) * x_Lon_Weight(i) * z_Rad_Weight(k)
+         enddo
+      enddo
+    end function IntLonRad_xz
+
+    !----(ϥǡ z)---
+    function IntRad_z(z)  ! ưʬ
+      real(8), dimension(0:km), intent(in) :: z    ! 2 ʻǡ
+      real(8)                              :: IntRad_z   ! ٳʻǡ
+      integer :: k
+
+      IntRad_z = 0.0d0
+      do k=0,km
+         IntRad_z = IntRad_z + z(k) * z_Rad_Weight(k) 
+      enddo
+    end function IntRad_z
+
+  !--------------- ʿѷ׻ -----------------
+    !----(ϥǡ xyz)---
     function yz_AvrLon_xyz(xyz)  ! (Ӿ)ʬ
       real(8), dimension(im,jm,0:km), intent(in) :: xyz    ! 3 ʻǡ
       real(8), dimension(jm,0:km)  :: yz_AvrLon_xyz        ! Ҹ̳ʻǡ
@@ -422,6 +614,64 @@ module wt_module
 
     end function AvrLonLatRad_xyz
 
+    !----(ϥǡ yz)---
+    function z_AvrLat_yz(yz)  ! ʬ
+      real(8), dimension(jm,0:km), intent(in) :: yz   ! 2 ʻǡ
+      real(8), dimension(0:km)  :: z_AvrLat_yz        ! ư³ʻǡ
+
+      z_AvrLat_yz = z_IntLat_yz(yz)/sum(y_Lat_Weight)
+    end function z_AvrLat_yz
+
+    function y_AvrRad_yz(yz)  ! ưʬ
+      real(8), dimension(jm,0:km), intent(in) :: yz    ! 2 ʻǡ
+      real(8), dimension(jm)  :: y_AvrRad_yz           ! ٱ߳ʻǡ
+
+      y_AvrRad_yz = y_IntRad_yz(yz)/sum(z_Rad_Weight)
+
+    end function y_AvrRad_yz
+
+    function AvrLatRad_yz(yz)  ! ư(Ҹ)ʬ
+      real(8), dimension(jm,0:km), intent(in) :: yz  ! Ҹ̳ʻǡ
+      real(8)                   :: AvrLatRad_yz      ! ʬ
+
+      AvrLatRad_yz = IntLatRad_yz(yz)/(sum(y_Lat_Weight)*sum(z_Rad_Weight))
+
+    end function AvrLatRad_yz
+
+    !----(ϥǡ xz)---
+    function z_AvrLon_xz(xz)  ! (Ӿ)ʬ
+      real(8), dimension(im,0:km), intent(in) :: xz   ! 2 ʻǡ
+      real(8), dimension(0:km)  :: z_AvrLon_xz        ! ľʻǡ
+
+      z_AvrLon_xz = z_IntLon_xz(xz)/sum(x_Lon_Weight)
+
+    end function z_AvrLon_xz
+
+    function x_AvrRad_xz(xz)  ! ưʬ
+      real(8), dimension(im,0:km), intent(in) :: xz    ! 2 ʻǡ
+      real(8), dimension(im)  :: x_AvrRad_xz           ! ٳʻǡ
+
+      x_AvrRad_xz = x_IntRad_xz(xz)/sum(z_Rad_Weight)
+
+    end function x_AvrRad_xz
+
+    function AvrLonRad_xz(xz)  ! ư(ٱ)ʬ
+      real(8), dimension(im,0:km), intent(in) :: xz    ! 2 ʻǡ
+      real(8)                                 :: AvrLonRad_xz      ! ʬ
+
+      AvrLonRad_xz = IntLonRad_xz(xz)/(sum(x_Lon_Weight)*sum(z_Rad_Weight))
+
+    end function AvrLonRad_xz
+
+    !----(ϥǡ z)---
+    function AvrRad_z(z)  ! ưʬ
+      real(8), dimension(im,0:km), intent(in) :: z    ! 2 ʻǡ
+      real(8)                                 :: AvrRad_z   ! ٳʻǡ
+
+      AvrRad_z = IntRad_z(z)/sum(z_Rad_Weight)
+
+    end function AvrRad_z
+
   !--------------- ݥ/ȥǥʬ -----------------
 
     function wt_KxRGrad_wt(wt)    ! kr = /ߦ
@@ -490,6 +740,10 @@ module wt_module
     subroutine wt_Potential2Vector(&
          xyz_VLON,xyz_VLAT,xyz_VRAD,wt_TORPOT,wt_POLPOT)
 
+      ! ȥݥݥƥ󥷥릷,ɽȯ٥ȥ
+      !     v = x(r) + xx(r) 
+      ! γʬ׻
+
       real(8), dimension(im,jm,0:km)     :: xyz_VLON   ! ٥ȥ(ʬ)
       real(8), dimension(im,jm,0:km)     :: xyz_VLAT   ! ٥ȥ(ʬ)
       real(8), dimension(im,jm,0:km)     :: xyz_VRAD   ! ٥ȥ(ưʬ)
@@ -506,35 +760,155 @@ module wt_module
 
     end subroutine wt_Potential2Vector
 
+    subroutine wt_Potential2Rotation(&
+       xyz_RotVLON,xyz_RotVLAT,xyz_RotVRAD,wt_TORPOT,wt_POLPOT)
+
+      ! ȥݥݥƥ󥷥릷,ɽȯ٥ȥ
+      !     v = x(r) + xx(r) 
+      ! Ф, βž
+      !     xv = xx(r) + xxx(r) = xx(r) - x((^2)r)
+      ! ׻. 
+
+      ! ٥ȥβž
+      real(8), dimension(im,jm,0:km), intent(OUT) :: xyz_RotVLON   ! ʬ
+      real(8), dimension(im,jm,0:km), intent(OUT) :: xyz_RotVLAT   ! ʬ
+      real(8), dimension(im,jm,0:km), intent(OUT) :: xyz_RotVRAD   ! ưʬ
+
+      ! ϥ٥ȥɽݥƥ󥷥
+      real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) &
+                                         :: wt_TORPOT ! ȥݥƥ󥷥
+      real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) &
+                                         :: wt_POLPOT ! ݥݥƥ󥷥
+
+      call wt_Potential2Vector( &
+           xyz_RotVLON,xyz_RotVLAT,xyz_RotVRAD, &
+           -wt_Lapla_wt(wt_POLPOT), wt_TORPOT)
+
+    end subroutine wt_Potential2Rotation
+
  !------------------- ׻ ----------------------
-  subroutine wt_VGradV(xyz_VGRADV_LON,xyz_VGRADV_LAT,xyz_VGRADV_RAD, &
-                       xyz_VLON,xyz_VLAT,xyz_VRAD )
+    subroutine wt_VGradV(xyz_VGRADV_LON,xyz_VGRADV_LAT,xyz_VGRADV_RAD, &
+                          xyz_VLON,xyz_VLAT,xyz_VRAD )
 
-    real(8), dimension(im,jm,0:km),intent(out)   :: xyz_VGRADV_LON
-    real(8), dimension(im,jm,0:km),intent(out)   :: xyz_VGRADV_LAT
-    real(8), dimension(im,jm,0:km),intent(out)   :: xyz_VGRADV_RAD
-    real(8), dimension(im,jm,0:km),intent(in)    :: xyz_VLON
-    real(8), dimension(im,jm,0:km),intent(in)    :: xyz_VLAT
-    real(8), dimension(im,jm,0:km),intent(in)    :: xyz_VRAD
+      real(8), dimension(im,jm,0:km),intent(out)   :: xyz_VGRADV_LON
+      real(8), dimension(im,jm,0:km),intent(out)   :: xyz_VGRADV_LAT
+      real(8), dimension(im,jm,0:km),intent(out)   :: xyz_VGRADV_RAD
+      real(8), dimension(im,jm,0:km),intent(in)    :: xyz_VLON
+      real(8), dimension(im,jm,0:km),intent(in)    :: xyz_VLAT
+      real(8), dimension(im,jm,0:km),intent(in)    :: xyz_VRAD
 
-    xyz_VGRADV_LON = xyz_wt(    &
-            wt_Div_xyz_xyz_xyz( &
-                xyz_VLON * xyz_VLON, xyz_VLON*xyz_VLAT, xyz_VLON*xyz_VRAD )) &
-          + xyz_VLON*xyz_VRAD/xyz_RAD              &
-          - xyz_VLON*xyz_VLAT*tan(xyz_LAT)/xyz_RAD 
+      xyz_VGRADV_LON = &
+              xyz_Div_xyz_xyz_xyz( &
+                  xyz_VLON * xyz_VLON, xyz_VLON*xyz_VLAT, xyz_VLON*xyz_VRAD ) &
+            + xyz_VLON*xyz_VRAD/xyz_RAD              &
+            - xyz_VLON*xyz_VLAT*tan(xyz_LAT)/xyz_RAD 
 
-    xyz_VGRADV_LAT = xyz_wt(    &
-            wt_Div_xyz_xyz_xyz( &
-                xyz_VLAT*xyz_VLON, xyz_VLAT*xyz_VLAT, xyz_VLAT*xyz_VRAD )) &
-          + xyz_VLAT*xyz_VRAD/xyz_RAD        &
-          + xyz_VLON**2*tan(xyz_LAT)/xyz_RAD 
+      xyz_VGRADV_LAT = &
+              xyz_Div_xyz_xyz_xyz( &
+                  xyz_VLAT*xyz_VLON, xyz_VLAT*xyz_VLAT, xyz_VLAT*xyz_VRAD ) &
+            + xyz_VLAT*xyz_VRAD/xyz_RAD        &
+            + xyz_VLON**2*tan(xyz_LAT)/xyz_RAD 
 
-    xyz_VGRADV_RAD = xyz_wt(    &
-            wt_Div_xyz_xyz_xyz( &
-                xyz_VRAD*xyz_VLON, xyz_VRAD*xyz_VLAT, xyz_VRAD*xyz_VRAD )) &
-          - (xyz_VLON**2 + xyz_VLAT**2)/xyz_RAD 
+      xyz_VGRADV_RAD = &
+              xyz_Div_xyz_xyz_xyz( &
+                  xyz_VRAD*xyz_VLON, xyz_VRAD*xyz_VLAT, xyz_VRAD*xyz_VRAD ) &
+            - (xyz_VLON**2 + xyz_VLAT**2)/xyz_RAD 
+
+    end subroutine wt_VGradV
+
+  !--------------- ݥ/ȥǥѥڥȥ ----------------
+
+    function nmz_ToroidalEnergySpectrum_wt(wt_TORPOT)
+      real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) &
+           :: wt_TORPOT                       ! ȥݥƥ󥷥
+      real(8), dimension(0:nm,-nm:nm,0:km) &
+           :: nmz_ToroidalEnergySpectrum_wt   ! ڥȥȥʬ
+
+      real(8), dimension((nm+1)*(nm+1),0:lm) ::wz_DATA   ! ΰ
+      integer :: n, m
+
+      nmz_ToroidalEnergySpectrum_wt = wt_VMiss
+
+      wz_DATA = wz_wt(wt_TORPOT)
+      do n=0,nm
+         do m=-n,n
+            nmz_ToroidalEnergySpectrum_wt(n,m,:) &
+              = 0.5 * n*(n+1)* (4*pi) * z_Rad**2 &
+                * wz_DATA(l_nm(n,m),:)**2
+         enddo
+      enddo
+
+    end function nmz_ToroidalEnergySpectrum_wt
+
+    function nz_ToroidalEnergySpectrum_wt(wt_TORPOT)
+      real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) &
+           :: wt_TORPOT                      ! ȥݥƥ󥷥
+      real(8), dimension(0:nm,0:km) &
+           :: nz_ToroidalEnergySpectrum_wt   ! ڥȥȥʬ
+
+      real(8), dimension((nm+1)*(nm+1),0:km) ::wz_DATA   ! ΰ
+      integer :: n, m
+
+      wz_DATA = wz_wt(wt_TORPOT)
+      do n=0,nm
+         nz_ToroidalEnergySpectrum_wt(n,:) &
+              = 0.5 * n*(n+1)* (4*pi) * z_Rad**2 &
+                * sum(wz_Data(l_nm(n,(/(m,m=-n,n)/)),:)**2,1)
+      enddo
+
+    end function nz_ToroidalEnergySpectrum_wt
+
+    function nmz_PoloidalEnergySpectrum_wt(wt_POLPOT)
+      real(8), dimension((nm+1)*(nm+1),0:km), intent(in) &
+           :: wt_POLPOT                      ! ݥݥƥ󥷥
+      real(8), dimension(0:nm,-nm:nm,0:km) &
+           :: nmz_PoloidalEnergySpectrum_wt   ! ڥȥȥʬ
+
+
+      real(8), dimension((nm+1)*(nm+1),0:km) ::wz_DATA1   ! ΰ
+      real(8), dimension((nm+1)*(nm+1),0:km) ::wz_DATA2   ! ΰ
+      integer :: n, m
+
+      nmz_PoloidalEnergySpectrum_wt = wt_VMiss
+
+      wz_Data1 = wz_wt(wt_POLPOT)
+      wz_Data2 = wz_Rad*wz_wt(wt_DRad_wt(wt_POLPOT)) &    ! d(r)/dr
+               + wz_wt(wt_POLPOT)                         ! = rd/dr+
+
+      do n=0,nm
+         do m=-n,n
+            nmz_PoloidalEnergySpectrum_wt(n,m,:) = &
+                 + 0.5* n*(n+1)* (4*pi) &
+                 *( wz_Data2(l_nm(n,m),:)**2  &
+                   + n*(n+1)*wz_Data1(l_nm(n,m),:)**2 )
+         enddo
+      enddo
+
+    end function nmz_PoloidalEnergySpectrum_wt
+
+    function nz_PoloidalEnergySpectrum_wt(wt_POLPOT)
+      real(8), dimension((nm+1)*(nm+1),0:lm), intent(in) &
+           :: wt_POLPOT                      ! ݥݥƥ󥷥
+      real(8), dimension(0:nm,0:km) &
+           :: nz_PoloidalEnergySpectrum_wt   ! ڥȥȥʬ
+
+      real(8), dimension((nm+1)*(nm+1),0:km) ::wz_DATA1   ! ΰ
+      real(8), dimension((nm+1)*(nm+1),0:km) ::wz_DATA2   ! ΰ
+      integer :: n, m
+
+      wz_Data1 = wz_wt(wt_POLPOT)
+      wz_Data2 = wz_Rad*wz_wt(wt_DRad_wt(wt_POLPOT)) &    ! d(r)/dr
+               + wz_wt(wt_POLPOT)                         ! = rd/dr+
+
+      do n=0,nm
+         nz_PoloidalEnergySpectrum_wt(n,:) = &
+              + 0.5* n*(n+1)* (4*pi) &
+              *( sum(wz_Data2(l_nm(n,(/(m,m=-n,n)/)),:)**2,1)  &
+                + n*(n+1)*sum(wz_Data1(l_nm(n,(/(m,m=-n,n)/)),:)**2,1) )
+      enddo
+
+    end function nz_PoloidalEnergySpectrum_wt
 
-  end subroutine wt_VGradV
 
   !---------------  -----------------
 
@@ -589,7 +963,7 @@ module wt_module
             call at_BoundariesTau_DD(wt)
          endif
       case default
-         call msgdmp('E','wt_BoundariesTau','B.C. not supported')
+         call MessageNotify('E','wt_BoundariesTau','B.C. not supported')
       end select
 
     end subroutine wt_BoundariesTau
@@ -645,27 +1019,32 @@ module wt_module
             call at_BoundariesGrid_DD(wt)
          endif
       case default
-         call msgdmp('E','wt_BoundariesGrid','B.C. not supported')
+         call MessageNotify('E','wt_BoundariesGrid','B.C. not supported')
       end select
 
     end subroutine wt_BoundariesGrid
 
-    subroutine wt_TorBoundariesTau(wt_TORPOT,cond) ! ȥݥƥ󥷥붭
+    subroutine wt_TorBoundariesTau(wt_TORPOT,values,cond) 
+      ! ȥݥƥ󥷥붭
       ! Chebyshev ֤ǤζŬ
       !  ȥݥƥ󥷥 \psi ζŬ
       ! Ǵ
-      !    \psi = 0  at boundaries
+      !    \psi = values  at boundaries (default 0)
       ! Ϥʤ
       !    \DP{\psi/r}{r} = 0  at boundaries
 
       real(8), dimension((nm+1)*(nm+1),0:lm),intent(inout)      :: wt_TORPOT
               ! ŬѤǡ. 줿֤ͤ. 
 
+      real(8), dimension((nm+1)*(nm+1),2), intent(in), optional :: values
+              ! ξüǤΥȥݥƥ󥷥
+              ! ǴλΤͭ
+
       character(len=2), intent(in), optional  :: cond
               ! 凉å. ά 'RR'
-              !   RR : ξüǴ
-              !   RF,FR : Ǵ/Ϥʤ
-              !   FF : ξüϤʤ
+              !   RR    : ξüǴ
+              !   RF/FR : Ǵ/Ϥʤ
+              !   FF    : ξüϤʤ
 
       real(8), dimension(:,:), allocatable  :: alu
       integer, dimension(:), allocatable    :: kp
@@ -690,7 +1069,7 @@ module wt_module
          case ('FF')
             rigid1 = .FALSE. ; rigid2 = .FALSE.
          case default
-            call msgdmp('E','wt_TorBoundariesTau','B.C. not supported')
+            call MessageNotify('E','wt_TorBoundariesTau','B.C. not supported')
          end select
       endif
 
@@ -712,7 +1091,7 @@ module wt_module
             tg_data = az_at(at_dr_at(at_az( &
                  az_at(tt_data)/spread(z_rad,1,lm+1))))
          endif
-         alu(lm-1,:) = tg_data(:,0)
+         alu(lm-1,:) = tg_data(:,0)       !  k=0 ǤξＰ
 
          if ( rigid2 ) then
             tg_data = az_at(tt_data)    
@@ -720,34 +1099,69 @@ module wt_module
             tg_data = az_at(at_dr_at(at_az( &
                  az_at(tt_data)/spread(z_rad,1,lm+1))))
          endif
-         alu(lm,:)   = tg_data(:,km)
+         alu(lm,:)   = tg_data(:,km)      !  k=km ǤξＰ
 
          call ludecomp(alu,kp)
+
+         if ( rigid1 .AND. present(values) ) then 
+            call MessageNotify('M','wt_TorBoundariesTau',&
+                 'Toroidal potential at k=0 was given by the optional variable.')
+         else if ( rigid1 .AND. (.NOT.present(values)) ) then
+            call MessageNotify('M','wt_TorBoundariesTau',&
+                 'Toroidal potential at k=0 was set to zero.')
+         else if ( (.NOT. rigid1) .AND. present(values) ) then
+            call MessageNotify('W','wt_TorBoundariesTau',&
+                 'Boundary value k=0 cannot be set under stress-free condition.')
+         endif
+
+         if ( rigid2 .AND. present(values) ) then 
+            call MessageNotify('M','wt_TorBoundariesTau',&
+                 'Toroidal potential at k=0 was given by the optional variable.')
+         else if ( rigid2 .AND. (.NOT.present(values)) ) then
+            call MessageNotify('M','wt_TorBoundariesTau',&
+                 'Toroidal potential at k=0 was set to zero.')
+         else if ( (.NOT. rigid2) .AND. present(values) ) then
+            call MessageNotify('W','wt_TorBoundariesTau',&
+                 'Boundary value k=0 cannot be set under stress-free condition.')
+         endif
+      endif
+
+      if ( rigid1 .AND. present(values) ) then
+         wt_torpot(:,lm-1) = values(:,1)
+      else
+         wt_torpot(:,lm-1) = 0
+      endif
+      if ( rigid2 .AND. present(values) ) then
+         wt_torpot(:,lm)   = values(:,2)
+      else
+         wt_torpot(:,lm) = 0
       endif
 
-      wt_torpot(:,lm-1) = 0
-      wt_torpot(:,lm)   = 0
       wt_torpot = lusolve(alu,kp,wt_TORPOT)
 
     end subroutine wt_TorBoundariesTau
 
-    subroutine wt_TorBoundariesGrid(wt_TORPOT,cond) ! ȥݥƥ󥷥붭
+    subroutine wt_TorBoundariesGrid(wt_TORPOT,values,cond) 
+      ! ȥݥƥ󥷥붭
       ! ¶֤ǤζŬ
       !  ȥݥƥ󥷥 \psi ζŬ
       ! Ǵ
-      !    \psi = 0  at boundaries
+      !    \psi = values  at boundaries (default 0)
       ! Ϥʤ
       !    \DP{\psi/r}{r} = 0  at boundaries
 
       real(8), dimension((nm+1)*(nm+1),0:lm),intent(inout)      :: wt_TORPOT
               ! ŬѤǡ. 줿֤ͤ. 
 
+      real(8), dimension((nm+1)*(nm+1),2), intent(in), optional :: values
+              ! ξüǤΥȥݥƥ󥷥
+              ! ǴλΤͭ
+
       character(len=2), intent(in), optional  :: cond
               ! 凉å. ά 'RR'
-              !   RR : ξüǴ
+              !   RR    : ξüǴ
               !   RF,FR : Ǵ/Ϥʤ
-              !   FF : ξüϤʤ
-
+              !   FF    : ξüϤʤ
 
       real(8), dimension((nm+1)*(nm+1),0:km):: wz_TORPOT
       real(8), dimension(:,:), allocatable  :: alu
@@ -773,7 +1187,7 @@ module wt_module
          case ('FF')
             rigid1 = .FALSE. ; rigid2 = .FALSE.
          case default
-            call msgdmp('E','wt_TorBoundariesGrid','B.C. not supported')
+            call MessageNotify('E','wt_TorBoundariesGrid','B.C. not supported')
          end select
       endif
 
@@ -781,7 +1195,7 @@ module wt_module
          first = .false.
 
          if ( lm /= km ) then
-            call msgdmp('E','TorBoundariesGrid', &
+            call MessageNotify('E','TorBoundariesGrid', &
              'Chebyshev truncation and number of grid points should be same.')
          endif
 
@@ -795,26 +1209,59 @@ module wt_module
          alu = transpose(tz_data)       ! ΰǤͤϤΤޤ
 
          if ( rigid1 ) then
-            tz_data = az_at(tt_data)    ! ǴǤ 0
+            tz_data = az_at(tt_data)
          else
             tz_data = az_at(at_dr_at(at_az( &
                  az_at(tt_data)/spread(z_rad,1,lm+1))))
          endif
-         alu(0,:) = tz_data(:,0)
+         alu(0,:) = tz_data(:,0)        !  k=0 ǤξＰ
 
          if ( rigid2 ) then
-            tz_data = az_at(tt_data)    ! ǴǤ 0
+            tz_data = az_at(tt_data)
          else
             tz_data = az_at(at_dr_at(at_az( &
                  az_at(tt_data)/spread(z_rad,1,lm+1))))
          endif
-         alu(km,:)   = tz_data(:,km)
+         alu(km,:)   = tz_data(:,km)    !  k=km ǤξＰ
 
          call ludecomp(alu,kp)
+
+         if ( rigid1 .AND. present(values) ) then 
+            call MessageNotify('M','wt_TorBoundariesGrid',&
+                 'Toroidal potential at k=0 was given by the optional variable.')
+         else if ( rigid1 .AND. (.NOT.present(values)) ) then
+            call MessageNotify('M','wt_TorBoundariesGrid',&
+                 'Toroidal potential at k=0 was set to zero.')
+         else if ( (.NOT. rigid1) .AND. present(values) ) then
+            call MessageNotify('W','wt_TorBoundariesGrid',&
+                 'Boundary value at k=0 cannot be set under stress-free condition.')
+         endif
+
+         if ( rigid2 .AND. present(values) ) then 
+            call MessageNotify('M','wt_TorBoundariesGrid',&
+                 'Toroidal potential at k=km was given by the optional variable.')
+         else if ( rigid2 .AND. (.NOT.present(values)) ) then
+            call MessageNotify('M','wt_TorBoundariesGrid',&
+                 'Toroidal potential at k=km was set to zero.')
+         else if ( (.NOT. rigid2) .AND. present(values) ) then
+            call MessageNotify('W','wt_TorBoundariesGrid',&
+                 'Boundary value at k=km cannot be set under stress-free condition.')
+         endif
       endif
+
       wz_TorPot       = wz_wt(wt_TorPot)
-      wz_TorPot(:,0)  = 0
-      wz_TorPot(:,km) = 0
+
+      if ( rigid1 .AND. present(values) ) then
+         wz_TorPot(:,0)  = values(:,1)
+      else
+         wz_TorPot(:,0)  = 0
+      endif
+
+      if ( rigid2 .AND. present(values) ) then
+         wz_TorPot(:,km) = values(:,2)
+      else
+         wz_TorPot(:,km) = 0
+      endif
 
       wt_torpot = lusolve(alu,kp,wz_TorPot)
 
@@ -870,7 +1317,7 @@ module wt_module
          case ('FF')
             rigid1 = .FALSE. ; rigid2 = .FALSE.
          case default
-            call msgdmp('E','wt_laplapol2pol_wt','B.C. not supported')
+            call MessageNotify('E','wt_laplapol2pol_wt','B.C. not supported')
          end select
       endif
 
@@ -878,7 +1325,7 @@ module wt_module
          first = .false.
 
          if ( lm /= km ) then
-            call msgdmp('E','wz_LaplaPol2Pol_wz', &
+            call MessageNotify('E','wz_LaplaPol2Pol_wz', &
              'Chebyshev truncation and number of grid points should be same.')
          endif
 
@@ -985,7 +1432,7 @@ module wt_module
          case ('FF')
             rigid1 = .FALSE. ; rigid2 = .FALSE.
          case default
-            call msgdmp('E','wt_LaplaPol2PolGrid_wt','B.C. not supported')
+            call MessageNotify('E','wt_LaplaPol2PolGrid_wt','B.C. not supported')
          end select
       endif
 
@@ -993,7 +1440,7 @@ module wt_module
          first = .false.
 
          if ( lm /= km ) then
-            call msgdmp('E','wt_LaplaPol2PolGrid_wt', &
+            call MessageNotify('E','wt_LaplaPol2PolGrid_wt', &
              'Chebyshev truncation and number of grid points should be same.')
          endif
 
@@ -1136,7 +1583,7 @@ module wt_module
          first = .false.
 
          if ( lm /= km ) then
-            call msgdmp('E','TorMagBoundariesGrid', &
+            call MessageNotify('E','TorMagBoundariesGrid', &
              'Chebyshev truncation and number of grid points should be same.')
          endif
 
@@ -1253,7 +1700,7 @@ module wt_module
          first = .false.
 
          if ( lm /= km ) then
-            call msgdmp('E','PolMagBoundariesGrid', &
+            call MessageNotify('E','PolMagBoundariesGrid', &
              'Chebyshev truncation and number of grid points should be same.')
          endif
 
